package org.kantega.jexmec.store.xml;

/*
 * Copyright 2009 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.Plugin;
import org.kantega.jexmec.store.PluginStore;
import org.kantega.jexmec.store.PluginStoreProvider;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.File;
import java.io.IOException;

/**
 * PluginStoreProvider backed by a single XML file.
 */
public class XmlPluginStoreProvider implements PluginStoreProvider {
    private Document document;
    private File file;
    private static final String UID_ATTR = "uid";
    private static final String PLUGIN_ELEM = "plugin";


    public XmlPluginStoreProvider(File file) {
        this.file = file;
    }

    private synchronized Document parseOrCreateDocument(File file) {
        if(document == null) {
            try {
                DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
                if(file.exists()) {
                    return document = builder.parse(file);
                } else {
                    return document = builder.newDocument();

                }
            } catch (ParserConfigurationException e) {
                throw new RuntimeException(e);
            } catch (IOException e) {
                throw new RuntimeException(e);
            } catch (SAXException e) {
                throw new RuntimeException(e);
            }
        } else {
            return document;
        }
    }


    public synchronized PluginStore getStore(Plugin plugin) {
        final String pluginUid = plugin.getPluginUid();
        if(pluginUid == null) {
            throw new IllegalArgumentException("Plugin " + plugin + " returned null from getPluginUid");
        }
        Document document = parseOrCreateDocument(file);
        if(document.getChildNodes().getLength() == 0) {
            document.appendChild(document.createElement("properties"));
        }
        Element pluginElement = null;

        for(int i = 0; i < document.getDocumentElement().getChildNodes().getLength(); i++) {
            Node item = document.getDocumentElement().getChildNodes().item(i);
            if(item instanceof Element) {
                Element elem = (Element) item;
                final String id = elem.getAttribute(UID_ATTR);
                if(elem.getNodeName().equals(PLUGIN_ELEM) && id != null && id.equals(plugin.getPluginUid())) {
                    pluginElement = elem;
                    break;
                }
            }
        }

        if(pluginElement == null) {
            pluginElement = document.createElement(PLUGIN_ELEM);
            pluginElement.setAttribute(UID_ATTR, plugin.getPluginUid());
            document.getDocumentElement().appendChild(pluginElement);
        }

        return new XmlPluginStore(pluginElement) {
            void propertySaved() {
                serialize();
            }
        };
    }

    private synchronized void serialize() {
        try {
            TransformerFactory.newInstance().newTransformer().transform(new DOMSource(document), new StreamResult(file));
        } catch (TransformerException e) {
            throw new RuntimeException(e);
        }
    }


}
